/*
 *  enlazar -- Wifi handling for the installer
 *
 *  Authored by Stefan Schmidt <stefan@openmoko.org>
 *
 *  Copyright (C) 2008 OpenMoko Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the license.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  Current Version: $Rev$ ($Date$) [$Author$]
 */

#include <stdlib.h>
#include <string.h>
#include <Ecore_Evas.h>
#include <Ecore_Data.h>
#include <Ecore.h>
#include <Edje.h>
#include <Ewl.h>
#include <E_Nm.h>
#include <dbus/dbus.h>

#include "debug.h"

struct NM_Manager{
	E_NM_Context *ctx;
	Ecore_List *devices;
	Ecore_List *aps;

	E_DBus_Callback_Func cb_devices;
	E_DBus_Callback_Func cb_aps;
};

/* FIXME: global */
int top = 50;

Ecore_List * network_list;
Ecore_List * network_blocks;

Ecore_Evas	*	ee;

void
cb_manager_get_devices(void *data, void *reply, DBusError *err)
{
  NM_Manager *app = data;
}

void
cb_manager_wifi_aps(void *data, void *reply, DBusError *err)
{
   NM_Manager *app = data;
   E_NM_Access_Point *ap = reply;

  ecore_list_append(network_list, ap);
}

void finish_cb(void *data, Evas_Object *obj, const char *emission, const char *source) {

	debug("called\n");
	exit(0);
}

void clear_networks(Evas_Object * network_block, void * data) {

	debug("called\n");
	evas_object_del(network_block);
}

void
text_input(Ewl_Widget *w, void *e, void *d)
{
	const char * password;

	debug("called\n");

	password = ewl_text_text_get(EWL_TEXT(w));
	printf("Given password: %s\n", password);
}

void password_cb(void *data, Evas_Object *obj, const char *emission, const char *source) {

	Evas		*	evas = data;
	Evas_Object	*	password_input, * network_select = obj, * eo;
	Evas_Coord		edje_w, edje_h;
	Ewl_Widget	*	password_entry, * emb, * hbox;

	debug("called\n");
	/* Remove the old layout from the canvas */
	ecore_list_for_each(network_blocks, (Ecore_For_Each) clear_networks, evas);
	ecore_list_destroy(network_blocks);

	password_input = edje_object_add(evas);
	edje_object_file_set(password_input, "enlazar.edj", "Wifi.password");

	evas_object_move(password_input, 0, 0);
	edje_object_size_min_get(password_input, &edje_w, &edje_h);
	evas_object_resize(password_input, edje_w, edje_h);
	evas_object_show(password_input);

	edje_object_signal_callback_add(password_input, "password", "ok", finish_cb, evas);
	edje_object_signal_callback_add(password_input, "password", "cancel", finish_cb, evas);

	/* Setup the ewl embedded widget */
	emb = ewl_embed_new();
	ewl_object_fill_policy_set(EWL_OBJECT(emb), EWL_FLAG_FILL_ALL);
	eo = ewl_embed_canvas_set(EWL_EMBED(emb), evas,
			  (void *) ecore_evas_software_x11_window_get(ee));
	ewl_embed_focus_set(EWL_EMBED(emb), TRUE);
	ewl_widget_show(emb);

	/* Swallow the embedded ewl widget into the edje */
	edje_object_part_swallow(password_input, "swallow", eo);
	evas_object_show(eo);

	hbox = ewl_hbox_new();
	ewl_object_fill_policy_set(EWL_OBJECT(hbox), EWL_FLAG_FILL_NONE |
						EWL_FLAG_FILL_HFILL);
	ewl_container_child_append(EWL_CONTAINER(emb), hbox);
	ewl_widget_show(hbox);

	password_entry = ewl_password_new();
	ewl_text_font_set(EWL_TEXT(password_entry), "LiberationSans-Bold");
	ewl_text_font_size_set(EWL_TEXT(password_entry), 48);
	ewl_text_color_set(EWL_TEXT(password_entry), 62, 62, 62, 255);
	ewl_container_child_append(EWL_CONTAINER(hbox), password_entry);
	ewl_callback_append(password_entry, EWL_CALLBACK_VALUE_CHANGED, text_input, NULL);
	ewl_widget_show(password_entry);
}

void print_networks(E_NM_Access_Point *ap, void * data) {

	Evas		*	evas = data;
	Evas_Object	*	network_block;
	Evas_Coord		edje_w, edje_h;
	char			formatted_strength[6];
	Edje_Message_Int msg;
	double			bar_len;

	debug("Network name: %s\n", ap->ssid);
	debug("Strength: %i\n", ap->strength);

	network_block = edje_object_add(evas);
	edje_object_file_set(network_block, "enlazar.edj", "Wifi.block");

	edje_object_part_text_set(network_block, "network.button.name", ap->ssid);

	bar_len = 3.4 * ap->strength;
	debug("bar length: %i\n", (int) bar_len);
	msg.val = (int) bar_len;
	edje_object_message_send(network_block, EDJE_MESSAGE_INT, 1, &msg);

	snprintf(formatted_strength, sizeof(formatted_strength)-1, "%i%%", ap->strength);
	debug("After first strcat: %s\n", formatted_strength);
	edje_object_part_text_set(network_block, "network.button.strength", formatted_strength);

	evas_object_move(network_block, 20, top);
	top += 81;
	edje_object_size_min_get(network_block, &edje_w, &edje_h);
	evas_object_resize(network_block, edje_w, edje_h);
	evas_object_show(network_block);

	edje_object_signal_callback_add(network_block, "network", "ok", password_cb, evas);

	/* Store all networck_blocks in a list for later reference */
	ecore_list_append(network_blocks, network_block);

	free(ap);
}

void network_cb(void *data, Evas_Object *obj, const char *emission, const char *source) {

	Evas		*	evas = data;
	Evas_Object	*	network_layout, * start_dialog = obj;
	Evas_Coord		edje_w, edje_h;

	debug("called\n");
	/* Remove the old layout from the canvas */
	evas_object_del(start_dialog);

	network_layout = edje_object_add(evas);
	edje_object_file_set(network_layout, "enlazar.edj", "Wifi.network");

	evas_object_move(network_layout, 0, 0);
	edje_object_size_min_get(network_layout, &edje_w, &edje_h);
	evas_object_resize(network_layout, edje_w, edje_h);
	evas_object_show(network_layout);

	network_blocks = ecore_list_new();
	ecore_list_init(network_blocks);
	ecore_list_append(network_blocks, network_layout);

	/* FIXME: Sort list for strength */
	ecore_list_for_each(network_list, (Ecore_For_Each) print_networks, evas);
	ecore_list_destroy(network_list);
}

void cancel_cb(void *data, Evas_Object *obj, const char *emission, const char *source) {

	Evas_Object	*	error_dialog = data, * start_dialog = obj;
	Evas_Coord		edje_w, edje_h;

	debug("called\n");
	/* Remove the old layout from the canvas */
	evas_object_del(start_dialog);

	evas_object_move(error_dialog, 0, 0);
	edje_object_size_min_get(error_dialog, &edje_w, &edje_h);
	evas_object_resize(error_dialog, edje_w, edje_h);

	edje_object_part_text_set(error_dialog, "dialog_message", "<error>ERROR</error><message>   Unable to connect</message>");

	evas_object_show(error_dialog);
}

int main(int args, char ** argv) {

	Evas		*	evas;
	Evas_Object	*	start_dialog, * error_dialog;
	Evas_Coord		edje_w, edje_h;
	NM_Manager *app;

	if (!ecore_init())
		return EXIT_FAILURE;
	if (!ecore_string_init())
		return EXIT_FAILURE;
	if (!ecore_evas_init())
		return EXIT_FAILURE;
	if (!edje_init())
		return EXIT_FAILURE;
	if (!ewl_init(&args, argv))
		return EXIT_FAILURE;
	if (!e_dbus_init())
		return EXIT_FAILURE;

	ee = ecore_evas_software_x11_new(NULL, 0,  0, 0, 0, 0);
	if (!ee)
		return EXIT_FAILURE;

	ecore_evas_title_set(ee, "Wifi connection menu");
	ecore_evas_borderless_set(ee, 1);

	evas = ecore_evas_get(ee);
	evas_font_path_append(evas, "fonts/");

	start_dialog = edje_object_add(evas);
	edje_object_file_set(start_dialog, "enlazar.edj", "Wifi.dialog");

	error_dialog = edje_object_add(evas);
	edje_object_file_set(error_dialog, "enlazar.edj", "Wifi.error");

	evas_object_move(start_dialog, 0, 0);
	edje_object_size_min_get(start_dialog, &edje_w, &edje_h);
	evas_object_resize(start_dialog, edje_w, edje_h);

	edje_object_part_text_set(start_dialog, "dialog_message", "<type>CONNECT</type><message>   to a Wi-Fi network?</message>");

	evas_object_show(start_dialog);

	edje_object_signal_callback_add(start_dialog, "start", "ok", network_cb, evas);
	edje_object_signal_callback_add(start_dialog, "start", "cancel", cancel_cb, error_dialog);
	edje_object_signal_callback_add(error_dialog, "error", "ok", finish_cb, NULL);

	ecore_evas_resize(ee, (int)edje_w, (int)edje_h);
	ecore_evas_show(ee);

	app = calloc(1, sizeof(NM_Manager));

	app->ctx = e_nm_new();
	if (!app->ctx) {
		printf("Error connecting to system bus. Is it running?\n");
		return 1;
	}

	network_list = ecore_list_new();
	ecore_list_init(network_list);

	e_nm_get_devices(app->ctx, cb_manager_get_devices, app);

	debug("Start ecore main loop\n");
	ecore_main_loop_begin();

	return 0;
}
